@@ -408,7 +408,7 @@ class GraspGroup():
         '''
         rotation = T[:3,:3]
         translation = T[:3,3]
-        self.translations = np.dot(rotation, self.translations.reshape((3,-1))).reshape((-1,3)) + translation # (-1, 3)
+        self.translations = np.dot(rotation, self.translations.T).T + translation # (-1, 3)
         self.rotation_matrices = np.matmul(rotation, self.rotation_matrices).reshape((-1, 3, 3)) # (-1, 9)
         return self
 
@@ -1073,4 +1073,4 @@ class RectGraspGroup():
         np.random.shuffle(shuffled_rect_grasp_group_array)
         shuffled_rect_grasp_group = RectGraspGroup()
         shuffled_rect_grasp_group.rect_grasp_group_array = copy.deepcopy(shuffled_rect_grasp_group_array[:numGrasp])
-        return shuffled_rect_grasp_group
\ No newline at end of file
+        return shuffled_rect_grasp_group
