@@ -434,7 +434,7 @@ class GraspNet():
         y2 = len(masky) - np.argmax(masky[::-1]) 
         return (x1, y1, x2, y2)
 
-    def loadScenePointCloud(self, sceneId, camera, annId, align=False, format = 'open3d', use_workspace = False, use_mask = True):
+    def loadScenePointCloud(self, sceneId, camera, annId, align=False, format = 'open3d', use_workspace = False, use_mask = True, use_inpainting = False):
         '''
         **Input:**
 
@@ -453,6 +453,8 @@ class GraspNet():
         - use_mask: bool of whether crop the point cloud use mask(z>0), only open3d 0.9.0 is supported for False option.
                     Only turn to False if you know what you are doing.
 
+        - use_inpainting: bool of whether inpaint the depth image for the missing information.
+
         **Output:**
 
         - open3d.geometry.PointCloud instance of the scene point cloud.
@@ -461,6 +463,11 @@ class GraspNet():
         '''
         colors = self.loadRGB(sceneId = sceneId, camera = camera, annId = annId).astype(np.float32) / 255.0
         depths = self.loadDepth(sceneId = sceneId, camera = camera, annId = annId)
+        if use_inpainting:
+            fault_mask = depths < 200
+            depths[fault_mask] = 0
+            inpainting_mask = (np.abs(depths) < 10).astype(np.uint8)
+            depths = cv2.inpaint(depths, inpainting_mask, 5, cv2.INPAINT_NS)
         intrinsics = np.load(os.path.join(self.root, 'scenes', 'scene_%04d' % sceneId, camera, 'camK.npy'))
         fx, fy = intrinsics[0,0], intrinsics[1,1]
         cx, cy = intrinsics[0,2], intrinsics[1,2]
@@ -478,7 +485,9 @@ class GraspNet():
         points_z = depths / s
         points_x = (xmap - cx) / fx * points_z
         points_y = (ymap - cy) / fy * points_z
-
+        # print(f'points_x.shape:{points_x.shape}')
+        # print(f'points_y.shape:{points_y.shape}')
+        # print(f'points_z.shape:{points_z.shape}')
         if use_workspace:
             (x1, y1, x2, y2) = self.loadWorkSpace(sceneId, camera, annId)
             points_z = points_z[y1:y2,x1:x2]
@@ -488,9 +497,13 @@ class GraspNet():
 
         mask = (points_z > 0)
         points = np.stack([points_x, points_y, points_z], axis=-1)
+        # print(f'points.shape:{points.shape}')
         if use_mask:
             points = points[mask]
             colors = colors[mask]
+        else:
+            points = points.reshape((-1, 3))
+            colors = colors.reshape((-1, 3))
         if align:
             points = transform_points(points, camera_pose)
         if format == 'open3d':
@@ -645,17 +658,9 @@ class GraspNet():
                 grasp_group.grasp_group_array = np.concatenate((grasp_group.grasp_group_array, obj_grasp_array))
             return grasp_group
         else:
-            import copy
             # 'rect'
-            # for rectangle grasp, collision labels and grasp labels are not necessray. 
-            ##################### OLD LABEL ################
-            ############### MODIFICATION NEEDED ############
-            rect_grasp_label = np.load(os.path.join(self.root,'scenes','scene_%04d' % sceneId,camera,'rect','%04d.npy' % annId))
-            mask = rect_grasp_label[:,5] >= (1.1 - fric_coef_thresh)
-            rect_grasp_label = rect_grasp_label[mask]
-            rect_grasp = RectGraspGroup()
-            rect_grasp.rect_grasp_group_array = copy.deepcopy(rect_grasp_label)
-            return rect_grasp
+            rect_grasps = RectGraspGroup(os.path.join(self.root,'scenes','scene_%04d' % sceneId,camera,'rect','%04d.npy' % annId))
+            return rect_grasps
 
     def loadData(self, ids=None, *extargs):
         '''
@@ -696,7 +701,7 @@ class GraspNet():
             scene_name = 'scene_'+str(sceneId).zfill(4)
             return (rgbPath, depthPath, segLabelPath, metaPath, rectLabelPath, scene_name,annId)
 
-    def showObjGrasp(self, objIds=[], numGrasp=10, th=0.5, saveFolder='save_fig', show=False):
+    def showObjGrasp(self, objIds=[], numGrasp=10, th=0.5, maxWidth=0.08, saveFolder='save_fig', show=False):
         '''
         **Input:**
 
@@ -706,6 +711,8 @@ class GraspNet():
 
         - th: threshold of the coefficient of friction.
 
+        - maxWidth: float, only visualize grasps with width<=maxWidth
+
         - saveFolder: string of the path to save the rendered image.
 
         - show: bool of whether to show the image.
@@ -723,7 +730,7 @@ class GraspNet():
         if not os.path.exists(saveFolder):
             os.mkdir(saveFolder)
         for obj_id in objIds:
-            visObjGrasp(self.root, obj_id, num_grasp=numGrasp,th=th, save_folder=saveFolder, show=show)
+            visObjGrasp(self.root, obj_id, num_grasp=numGrasp, th=th, max_width=maxWidth, save_folder=saveFolder, show=show)
 
     def showSceneGrasp(self, sceneId, camera = 'kinect', annId = 0, format = '6d', numGrasp = 20, show_object = True, coef_fric_thresh = 0.1):
         '''
@@ -739,7 +746,7 @@ class GraspNet():
 
         - numGrasp: int of the displayed grasp number, grasps will be randomly sampled.
 
-        - coef_fric_thresh: float of the friction coefficient of grasps. 
+        - coef_fric_thresh: float of the friction coefficient of grasps.
         '''
         if format == '6d':
             geometries = []
@@ -761,7 +768,7 @@ class GraspNet():
             cv2.waitKey(0)
             cv2.destroyAllWindows()
 
-    def show6DPose(self, sceneIds, saveFolder='save_fig', show=False):
+    def show6DPose(self, sceneIds, saveFolder='save_fig', show=False, perObj=False):
         '''
         **Input:**
 
@@ -771,6 +778,8 @@ class GraspNet():
 
         - show: bool of whether to show the image.
 
+        - perObj: bool, show grasps on each object
+
         **Output:**
         
         - No output but to save the rendered image and maybe show the result.
@@ -785,4 +794,4 @@ class GraspNet():
         for scene_id in sceneIds:
             scene_name = 'scene_'+str(scene_id).zfill(4)
             vis6D(self.root, scene_name, 0, self.camera,
-                  align_to_table=True, save_folder=saveFolder, show=show)
+                  align_to_table=True, save_folder=saveFolder, show=show, per_obj=perObj)
